#!/usr/bin/python3

# import logging
# logging.basicConfig(level = logging.DEBUG)

import pyvips

a = pyvips.Image.black(100, 100)
a = a.draw_circle(128, 50, 50, 20)
b = a.hough_circle(scale=1, min_radius=15, max_radius=25)
b.write_to_file("x.v")



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

a = pyvips.Image.new_from_file(sys.argv[1])

b = a.write_to_memory()

c = pyvips.Image.new_from_memory(b, a.width, a.height, a.bands, a.bandfmt)

c.write_to_file("x.v")



#!/usr/bin/python3

import sys
import pyvips

a = pyvips.Image.new_from_file(sys.argv[1])

try:
    orientation = a.get('exif-ifd0-Orientation')
    a.set('orientation', int(orientation.split()[0]))
except Exception:
    a.set('orientation', 0)

a.write_to_file(sys.argv[2])



#!/usr/bin/python3

import logging
import pyvips

logging.basicConfig(level=logging.DEBUG)

print('test Image')
image = pyvips.Image.new_from_file('/data/john/pics/k2.jpg')
print('image =', image)
print('image.width =', image.width)
print('\n''')



#!/usr/bin/env python3
import pyperf
import pyvips


def operation_call(loops):
    range_it = range(loops)

    t0 = pyperf.perf_counter()

    for loops in range_it:
        _ = pyvips.Operation.call('black', 10, 10)

    return pyperf.perf_counter() - t0


runner = pyperf.Runner()
runner.bench_time_func('Operation.call', operation_call)



#!/usr/bin/python3

# import logging
# logging.basicConfig(level = logging.DEBUG)

import pyvips

a = pyvips.Image.new_from_array([[1, 2, 3], [4, 5, 6], [7, 8, 9]], 8, 128)

print('scale =', a.get('scale'))
print('offset =', a.get('offset'))

a.write_to_file("x.v")



#!/usr/bin/python3

import sys
import pyvips

input_file = open(sys.argv[1], "rb")


def read_handler(size):
    return input_file.read(size)


def seek_handler(offset, whence):
    input_file.seek(offset, whence)
    return input_file.tell()


source = pyvips.SourceCustom()
source.on_read(read_handler)
source.on_seek(seek_handler)

output_file = open(sys.argv[2], "wb")


def write_handler(chunk):
    return output_file.write(chunk)


def finish_handler():
    output_file.close()


target = pyvips.TargetCustom()
target.on_write(write_handler)
target.on_finish(finish_handler)

image = pyvips.Image.new_from_source(source, '', access='sequential')
image.write_to_target(target, '.png')



#!/usr/bin/env python3

import pyperf
import pyvips


def vips_bench(loops):
    range_it = range(loops)

    t0 = pyperf.perf_counter()

    for loops in range_it:
        im = pyvips.Image.new_from_file("tmp/x.tif", access='sequential')

        im = im.crop(100, 100, im.width - 200, im.height - 200)
        im = im.reduce(1.0 / 0.9, 1.0 / 0.9, kernel='linear')
        mask = pyvips.Image.new_from_array([[-1, -1, -1],
                                            [-1, 16, -1],
                                            [-1, -1, -1]], scale=8)
        im = im.conv(mask, precision='integer')

        im.write_to_file("tmp/x2.tif")

    return pyperf.perf_counter() - t0


runner = pyperf.Runner()
runner.bench_time_func('vips bench', vips_bench)



# vim: set fileencoding=utf-8 :

import pyvips
from helpers import JPEG_FILE


class TestImage:
    def test_pagejoin(self):
        image = pyvips.Image.new_from_file(JPEG_FILE)
        many_page = image.pagejoin([image, image])
        assert many_page.get_page_height() == image.height
        assert many_page.height / many_page.get_page_height() == 3

    def test_pagesplit(self):
        image = pyvips.Image.new_from_file(JPEG_FILE)
        many_page = image.pagejoin([image, image])
        image_list = many_page.pagesplit()
        assert len(image_list) == 3



#!/usr/bin/python3

import sys
import xml.etree.ElementTree as ET

from pyvips import ffi, values_for_enum, vips_lib, \
    type_map, type_name, type_from_name

# This file generates enums.py -- the set of classes giving the permissible
# values for the pyvips enums. Run with something like:
#
#   ./gen-enums.py ~/GIT/libvips/libvips/Vips-8.0.gir > enums.py
#   mv enums.py ../pyvips

# The GIR file
root = ET.parse(sys.argv[1]).getroot()
namespace = {
    "goi": "http://www.gtk.org/introspection/core/1.0"
}

# find all the enumerations and make a dict for them
xml_enums = {}
for node in root.findall("goi:namespace/goi:enumeration", namespace):
    xml_enums[node.get('name')] = node


def remove_prefix(enum_str):
    prefix = 'Vips'

    if enum_str.startswith(prefix):
        return enum_str[len(prefix):]

    return enum_str


def generate_enums():
    # otherwise we're missing some enums
    vips_lib.vips_token_get_type()
    vips_lib.vips_saveable_get_type()
    vips_lib.vips_image_type_get_type()

    all_enums = []

    def add_enum(gtype, a, b):
        nickname = type_name(gtype)
        all_enums.append(nickname)

        type_map(gtype, add_enum)

        return ffi.NULL

    type_map(type_from_name('GEnum'), add_enum)

    for name in all_enums:
        gtype = type_from_name(name)
        python_name = remove_prefix(name)
        if python_name not in xml_enums:
            continue

        node = xml_enums[python_name]
        enum_doc = node.find("goi:doc", namespace)

        print('')
        print('')
        print(f'class {python_name}(object):')
        print(f'    """{python_name}.')
        if enum_doc is not None:
            print('')
            print(f'{enum_doc.text}')
        print('')
        print('Attributes:')
        print('')
        for value in values_for_enum(gtype):
            python_name = value.replace('-', '_')
            member = node.find(f"goi:member[@name='{python_name}']", namespace)
            member_doc = member.find("goi:doc", namespace)
            if member_doc is not None:
                text = member_doc.text
                print(f'    {python_name.upper()} (str): {text}')
                print('')
        print('    """')
        print('')

        for value in values_for_enum(gtype):
            python_name = value.replace('-', '_').upper()
            print(f'    {python_name} = \'{value}\'')


if __name__ == "__main__":
    print('# libvips enums -- this file is generated automatically')
    generate_enums()



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)
# pyvips.cache_set_trace(True)

a = pyvips.Image.new_from_file(sys.argv[1])

profile = a.get("icc-profile-data")

with open('x.icm', 'w') as f:
    f.write(profile)



#!/usr/bin/python3

import sys

import pyvips

im = pyvips.Image.new_from_file(sys.argv[1], access="sequential")

footer = pyvips.Image.black(im.width, 150)
left_text = pyvips.Image.text("left corner", dpi=300)
right_text = pyvips.Image.text("right corner", dpi=300)
footer = footer.insert(left_text, 50, 50)
footer = footer.insert(right_text, im.width - right_text.width - 50, 50)
footer = footer.ifthenelse(0, [255, 0, 0], blend=True)

im = im.insert(footer, 0, im.height, expand=True)

im.write_to_file(sys.argv[2])



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)
# pyvips.cache_set_trace(True)


def to_polar(image):
    """Transform image coordinates to polar.

    The image is transformed so that it is wrapped around a point in the
    centre. Vertical straight lines become circles or segments of circles,
    horizontal straight lines become radial spokes.
    """
    # xy image, origin in the centre, scaled to fit image to a circle
    xy = pyvips.Image.xyz(image.width, image.height)
    xy -= [image.width / 2.0, image.height / 2.0]
    scale = min(image.width, image.height) / float(image.width)
    xy *= 2.0 / scale

    index = xy.polar()

    # scale vertical axis to 360 degrees
    index *= [1, image.height / 360.0]

    return image.mapim(index)


def to_rectangular(image):
    """Transform image coordinates to rectangular.

    The image is transformed so that it is unwrapped from a point in the
    centre. Circles or segments of circles become vertical straight lines,
    radial lines become horizontal lines.
    """
    # xy image, vertical scaled to 360 degrees
    xy = pyvips.Image.xyz(image.width, image.height)
    xy *= [1, 360.0 / image.height]

    index = xy.rect()

    # scale to image rect
    scale = min(image.width, image.height) / float(image.width)
    index *= scale / 2.0
    index += [image.width / 2.0, image.height / 2.0]

    return image.mapim(index)


a = pyvips.Image.new_from_file(sys.argv[1])
a = to_polar(a)
a = to_rectangular(a)
a.write_to_file(sys.argv[2])



from __future__ import division

import logging

import pyvips
from pyvips import ffi, vips_lib

logger = logging.getLogger(__name__)


class TargetCustom(pyvips.Target):
    """An output target you can connect action signals to to implement
    behaviour.

    """

    def __init__(self):
        """Make a new target you can customise.

        You can pass this target to (for example) :meth:`write_to_target`.

        """

        target = ffi.cast('VipsTarget*', vips_lib.vips_target_custom_new())
        super(TargetCustom, self).__init__(target)

    def on_write(self, handler):
        """Attach a write handler.

        The interface is exactly as io.write(). The handler is given a
        bytes-like object to write, and should return the number of bytes
        written.

        """

        def interface_handler(buf):
            bytes_written = handler(buf)
            # py2 will often return None for bytes_written ... replace with
            # the length of the string
            if bytes_written is None:
                bytes_written = len(buf)

            return bytes_written

        self.signal_connect("write", interface_handler)

    def on_finish(self, handler):
        """Attach a finish handler.

        This optional handler is called at the end of write. It should do any
        cleaning up necessary.

        """

        self.signal_connect("finish", handler)


__all__ = ['TargetCustom']



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

a = pyvips.Image.new_from_file(sys.argv[1])

b = a.write_to_buffer(".jpg")

c = pyvips.Image.new_from_buffer(b, "")



#!/usr/bin/python3

import logging
import pyvips

logging.basicConfig(level=logging.DEBUG)
pyvips.cache_set_trace(True)

try:
    a = pyvips.Image.new_from_file("/home/john/pics/babe.poop")
except pyvips.Error as e:
    print(str(e))

a = pyvips.Image.new_from_file("/home/john/pics/babe.jpg")
b = pyvips.Image.new_from_file("/home/john/pics/k2.jpg")

print('a =', a)
print('b =', b)

out = pyvips.call("add", a, b)

print('out =', out)

out = a.add(b)

print('out =', out)

out = out.linear([1, 1, 1], [2, 2, 2])

out.write_to_file("x.v")



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

a = pyvips.Image.new_from_file(sys.argv[1])
ipct = a.get("ipct-data")
print("ipct = ", ipct.get())
a.remove("ipct-data")
a.write_to_file("x.jpg")



#!/usr/bin/python3

import sys
import pyvips

im = pyvips.Image.new_from_file(sys.argv[1], access="sequential")

text = pyvips.Image.text(sys.argv[3], width=500, dpi=300, align="centre")

# we make an overlay with a solid colour in the main image bands, and a faded
# version of the text mask as the overlay

# drop any alpha
if im.hasalpha():
    no_alpha = im.extract_band(0, n=im.bands - 1)
else:
    no_alpha = im

# colours have four parts in cmyk images
if im.bands == 4:
    text_colour = [0, 255, 0, 0]
elif im.bands == 3:
    text_colour = [255, 0, 0]
else:
    text_colour = 255

overlay = no_alpha.new_from_image(text_colour)
overlay = overlay.bandjoin((text * 0.5).cast("uchar"))

# position overlay at the bottom left, with a margin
im = im.composite(overlay, "over", x=100, y=im.height - text.height - 100)

im.write_to_file(sys.argv[2])



#!/usr/bin/python3

import sys
import pyvips

source = pyvips.Source.new_from_descriptor(sys.stdin.fileno())
image = pyvips.Image.new_from_source(source, "")
target = pyvips.Target.new_to_descriptor(sys.stdout.fileno())
image.write_to_target(target, ".jpg")



#!/usr/bin/python3

# watermark an image, but do it in a context-sensitive way, so we will write
# black if the area we are overlaying the text on is white, for example

# usage:
#   ./watermark_context.py ~/pics/k2.jpg x.jpg "hello there!"

import sys
import pyvips


# zero-excluding average of an image ... return as an array, with the average
# for each band
def avgze(image):
    # since we use histograms
    if image.format != "uchar" and image.format != "ushort":
        raise Exception("uchar and ushort images only")

    # take the histogram, and set the count for 0 pixels to 0, removing them
    histze = image.hist_find().insert(pyvips.Image.black(1, 1), 0, 0)

    # number of non-zero pixels in each band
    nnz = [histze[i].avg() * histze.width * histze.height
           for i in range(histze.bands)]

    # multiply by the identity function and we get the sum of non-zero
    # pixels ... for 16-bit images, we need a larger identity
    # function
    totalze = histze * pyvips.Image.identity(ushort=histze.width > 256)

    # find average value in each band
    avgze = [totalze[i].avg() * histze.width * histze.height / nnz[i]
             for i in range(totalze.bands)]

    return avgze


# find an opposing colour ... we split the density range (0 .. mx) into three:
# values in the bottom third move to the top, values in the top third move to
# the bottom, and values in the middle also move to the bottom
def oppose(value, mx):
    if value < mx / 3:
        # bottom goes up
        return mx / 3 - value + 2 * mx / 3
    elif value < 2 * mx / 3:
        # middle goes down
        return 2 * mx / 3 - value
    else:
        # top goes down
        return mx - value


im = pyvips.Image.new_from_file(sys.argv[1])

text = pyvips.Image.text(sys.argv[3], width=500, dpi=300, align="centre")
text = text.rotate(45)

# the position of the overlay in the image
left = 100
top = im.height - text.height - 100

# find the non-alpha image bands
if im.hasalpha():
    no_alpha = im.extract_band(0, n=im.bands - 1)
else:
    no_alpha = im

# the pixels we will render the overlay on top of
bg = no_alpha.crop(left, top, text.width, text.height)

# mask the background with the text, so all non-text areas become zero, and
# find the zero-excluding average
avg = avgze(text.ifthenelse(bg, 0))

# for each band, find the opposing value
mx = 255 if im.format == "uchar" else 65535
text_colour = [oppose(avg[i], mx) for i in range(len(avg))]

# make an overlay ... we put solid colour into the image and set a faded
# version of the text mask as the alpha
overlay = bg.new_from_image(text_colour)
overlay = overlay.bandjoin((text * 0.5).cast("uchar"))

# and composite that on to the original image
im = im.composite(overlay, "over", x=left, y=top)

im.write_to_file(sys.argv[2])



#!/usr/bin/python3

import logging
import sys

import pyvips

logging.basicConfig(level=logging.DEBUG)

# pyvips.cache_set_trace(True)

a = pyvips.Image.new_from_file(sys.argv[1])

x = a.erode([[128, 255, 128],
             [255, 255, 255],
             [128, 255, 128]])

x.write_to_file(sys.argv[2])



#!/usr/bin/python3

import pyvips


def progress_print(name, progress):
    print('{}:'.format(name))
    print('   run = {}'.format(progress.run))
    print('   eta = {}'.format(progress.eta))
    print('   tpels = {}'.format(progress.tpels))
    print('   npels = {}'.format(progress.npels))
    print('   percent = {}'.format(progress.percent))


def preeval_cb(image, progress):
    progress_print('preeval', progress)


def eval_cb(image, progress):
    progress_print('eval', progress)

    # you can kill computation if necessary
    if progress.percent > 50:
        image.set_kill(True)


def posteval_cb(image, progress):
    progress_print('posteval', progress)


image = pyvips.Image.black(1, 500)
image.set_progress(True)
image.signal_connect('preeval', preeval_cb)
image.signal_connect('eval', eval_cb)
image.signal_connect('posteval', posteval_cb)
image.avg()



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

pyvips.cache_set_trace(True)

a = pyvips.Image.new_from_file(sys.argv[1])
print(a.max())
print(a.max())



# vim: set fileencoding=utf-8 :
# test helpers
import os
import tempfile
import pytest

import pyvips

IMAGES = os.path.join(os.path.dirname(__file__), os.pardir, 'images')
JPEG_FILE = os.path.join(IMAGES, "sample.jpg")
WEBP_FILE = os.path.join(IMAGES, "sample.webp")


# an expanding zip ... if either of the args is a scalar or a one-element list,
# duplicate it down the other side
def zip_expand(x, y):
    # handle singleton list case
    if isinstance(x, list) and len(x) == 1:
        x = x[0]
    if isinstance(y, list) and len(y) == 1:
        y = y[0]

    if isinstance(x, list) and isinstance(y, list):
        return list(zip(x, y))
    elif isinstance(x, list):
        return [[i, y] for i in x]
    elif isinstance(y, list):
        return [[x, j] for j in y]
    else:
        return [[x, y]]


# run a 1-ary function on a thing -- loop over elements if the
# thing is a list
def run_fn(fn, x):
    if isinstance(x, list):
        return [fn(i) for i in x]
    else:
        return fn(x)


# make a temp filename with the specified suffix and in the
# specified directory
def temp_filename(directory, suffix):
    temp_name = next(tempfile._get_candidate_names())
    filename = os.path.join(directory, temp_name + suffix)

    return filename


# test for an operator exists
def have(name):
    return pyvips.type_find("VipsOperation", name) != 0


# use as @skip_if_no('jpegload')
def skip_if_no(operator_name):
    return pytest.mark.skipif(not have(operator_name),
                              reason=('no {}, skipping test'.
                                      format(operator_name)))


# run a 2-ary function on two things -- loop over elements pairwise if the
# things are lists
def run_fn2(fn, x, y):
    if isinstance(x, pyvips.Image) or isinstance(y, pyvips.Image):
        return fn(x, y)
    elif isinstance(x, list) or isinstance(y, list):
        return [fn(i, j) for i, j in zip_expand(x, y)]
    else:
        return fn(x, y)


# test a pair of things which can be lists for approx. equality
def assert_almost_equal_objects(a, b, threshold=0.0001, msg=''):
    # print 'assertAlmostEqualObjects %s = %s' % (a, b)
    assert all([pytest.approx(x, abs=threshold) == y
                for x, y in zip_expand(a, b)]), msg


# test a pair of things which can be lists for equality
def assert_equal_objects(a, b, msg=''):
    # print 'assertEqualObjects %s = %s' % (a, b)
    assert all([x == y for x, y in zip_expand(a, b)]), msg


# test a pair of things which can be lists for difference less than a
# threshold
def assert_less_threshold(a, b, diff):
    assert all([abs(x - y) < diff for x, y in zip_expand(a, b)])


# run a function on an image and on a single pixel, the results
# should match
def run_cmp(message, im, x, y, fn):
    a = im(x, y)
    v1 = fn(a)
    im2 = fn(im)
    v2 = im2(x, y)
    assert_almost_equal_objects(v1, v2, msg=message)


# run a function on an image,
# 50,50 and 10,10 should have different values on the test image
def run_image(message, im, fn):
    run_cmp(message, im, 50, 50, fn)
    run_cmp(message, im, 10, 10, fn)


# run a function on (image, constant), and on (constant, image).
# 50,50 and 10,10 should have different values on the test image
def run_const(message, fn, im, c):
    run_cmp(message, im, 50, 50, lambda x: run_fn2(fn, x, c))
    run_cmp(message, im, 50, 50, lambda x: run_fn2(fn, c, x))
    run_cmp(message, im, 10, 10, lambda x: run_fn2(fn, x, c))
    run_cmp(message, im, 10, 10, lambda x: run_fn2(fn, c, x))


# run a function on a pair of images and on a pair of pixels, the results
# should match
def run_cmp2(message, left, right, x, y, fn):
    a = left(x, y)
    b = right(x, y)
    v1 = fn(a, b)
    after = fn(left, right)
    v2 = after(x, y)
    assert_almost_equal_objects(v1, v2, msg=message)


# run a function on a pair of images
# 50,50 and 10,10 should have different values on the test image
def run_image2(message, left, right, fn):
    run_cmp2(message, left, right, 50, 50,
             lambda x, y: run_fn2(fn, x, y))
    run_cmp2(message, left, right, 10, 10,
             lambda x, y: run_fn2(fn, x, y))



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

a = pyvips.Image.new_from_file(sys.argv[1])


def should_equal(test, a, b):
    if abs(a - b) > 0.01:
        print('%s: seen %g and %g' % (test, a, b))
        sys.exit(1)


def bandsplit(a):
    return [a.extract_band(i) for i in range(0, a.bands)]


# test operator overloads

# addition
b = a + 12
should_equal('add constant', a.avg() + 12, b.avg())

b = a + [12, 0, 0]
x = map(lambda x: x.avg())
bandsplit(a)
y = map(lambda x: x.avg())
bandsplit(b)
x[0] += 12
should_equal('add multiband constant', sum(x), sum(y))

b = a + [12, 0, 0]
b = a + b
b = 12 + a
b = [12, 0, 0] + a

b = a - 12
b = a - [12, 0, 0]
b = a - b
b = 12 - a
b = [12, 0, 0] - a

b = a * 12
b = a * [12, 1, 1]
b = a * b
b = 12 * a
b = [12, 1, 1] * a

b = a / 12
b = a / [12, 1, 1]
b = 12 / a
b = [12, 1, 1] / a
b = a / b

b = a // 12
b = a // [12, 1, 1]
b = 12 // a
b = [12, 1, 1] // a
b = a // b

b = a % 12
b = a % [12, 1, 1]
b = a % b

b = a ** 12
b = a ** [12, 1, 1]
b = 12 ** a
b = [12, 1, 1] ** a
b = a ** b

b = a << 12
b = a << [12, 1, 1]
b = a << b

b = a >> 12
b = a >> [12, 1, 1]
b = a >> b

b = a & 12
b = a & [12, 1, 1]
b = 12 & a
b = [12, 1, 1] & a
b = a & b

b = a | 12
b = a | [12, 1, 1]
b = 12 | a
b = [12, 1, 1] | a
b = a | b

b = a ^ 12
b = a ^ [12, 1, 1]
b = 12 ^ a
b = [12, 1, 1] ^ a
b = a ^ b

b = -a
b = +a
b = abs(a)
b = ~a



#!/usr/bin/python3

import sys
import pyvips

pyvips.leak_set(True)
pyvips.cache_set_max(0)

for i in range(1000):
    print("loop {0} ...".format(i))
    im = pyvips.Image.new_from_file(sys.argv[1])
    im = im.embed(100, 100, 3000, 3000, extend="mirror")
    im.write_to_file("x.v")



# flake8: noqa

import pkgconfig
from cffi import FFI

# we must have the vips package to be able to do anything
if not pkgconfig.exists('vips'): 
    raise Exception('unable to find pkg-config package "vips"')
if pkgconfig.installed('vips', '< 8.2'):
    raise Exception('pkg-config "vips" is too old -- need libvips 8.2 or later')

ffibuilder = FFI()

ffibuilder.set_source("_libvips",
    r""" 
        #include <vips/vips.h>
    """, 
    **pkgconfig.parse('vips'))

# pkgconfig 1.5+ has modversion ... otherwise, use a small shim
try:
    from pkgconfig import modversion
except ImportError:
    def modversion(package):
        # will need updating once we hit 8.20 :(
        for i in range(20, 3, -1):
            if pkgconfig.installed(package, '>= 8.' + str(i)):
                # be careful micro version is always set to 0
                return '8.' + str(i) + '.0'
        return '8.2.0'

major, minor, micro = [int(s) for s in modversion('vips').split('.')]

features = {
    'major': major,
    'minor': minor,
    'micro': micro,
    'api': True,
}

import vdecls

# handy for debugging
#with open('vips-source.txt','w') as f:
#    c = vdecls.cdefs(features)
#    f.write(c)

ffibuilder.cdef(vdecls.cdefs(features))

if __name__ == "__main__":
    ffibuilder.compile(verbose=True)



#!/usr/bin/python3

import sys
import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

a = pyvips.Image.new_from_file(sys.argv[1])

b = pyvips.Image.new_from_file(sys.argv[2])

c = a.join(b, pyvips.Direction.HORIZONTAL,
           expand=True,
           shim=100,
           align=pyvips.Align.CENTRE,
           background=[128, 255, 128])

c.write_to_file(sys.argv[3])



# -*- coding: utf-8 -*-
#
# pyvips documentation build configuration file, created by
# sphinx-quickstart on Wed Aug  9 15:19:17 2017.
#
# This file is execfile()d with the current directory set to its
# containing dir.
#
# Note that not all possible configuration values are present in this
# autogenerated file.
#
# All configuration values have a default; values that are commented out
# serve to show the default.

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.

import os
import sys
import sphinx_rtd_theme
sys.path.insert(0, os.path.abspath('..'))


# -- General configuration ------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.
#
# needs_sphinx = '1.0'

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.autosummary',
    'sphinx.ext.mathjax',
    'sphinx.ext.viewcode',
    'sphinx.ext.githubpages',
    'sphinx.ext.napoleon',
]

autodoc_member_order = 'bysource'

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# The suffix(es) of source filenames.
# You can specify multiple suffix as a list of string:
#
# source_suffix = ['.rst', '.md']
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'pyvips'
copyright = u'2019, John Cupitt'
author = u'John Cupitt'

# The version info for the project you're documenting, acts as replacement for
# |version| and |release|, also used in various other places throughout the
# built documents.
#
# The short X.Y version.
version = u'2.1'
# The full version, including alpha/beta/rc tags.
release = u'2.1.16'

# The language for content autogenerated by Sphinx. Refer to documentation
# for a list of supported languages.
#
# This is also used if you do content translation via gettext catalogs.
# Usually you set "language" from the command line for these cases.
language = None

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This patterns also effect to html_static_path and html_extra_path
exclude_patterns = ['global.rst']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'sphinx'

# If true, `todo` and `todoList` produce output, else they produce nothing.
todo_include_todos = False


# -- Options for HTML output ----------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
html_theme = 'sphinx_rtd_theme'
html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]

# Theme options are theme-specific and customize the look and feel of a theme
# further.  For a list of options available for each theme, see the
# documentation.
#
# html_theme_options = {}

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# Custom sidebar templates, must be a dictionary that maps document names
# to template names.
#
# This is required for the alabaster theme
# refs: http://alabaster.readthedocs.io/en/latest/installation.html#sidebars
html_sidebars = {
    '**': [
        'about.html',
        'navigation.html',
        'searchbox.html',
    ]
}

html_context = {
    # Enable the "Edit on GitHub" link within the header of each page.
    'display_github': True,
    # Set the following variables to generate the resulting github URL for
    # each page.
    'github_user': 'libvips',
    'github_repo': 'pyvips',
    'github_version': 'master/doc/'
}

# -- Options for HTMLHelp output ------------------------------------------

# Output file base name for HTML help builder.
htmlhelp_basename = 'pyvipsdoc'


# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    #
    # 'papersize': 'letterpaper',

    # The font size ('10pt', '11pt' or '12pt').
    #
    # 'pointsize': '10pt',

    # Additional stuff for the LaTeX preamble.
    #
    # 'preamble': '',

    # Latex figure (float) alignment
    #
    # 'figure_align': 'htbp',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
    (master_doc, 'pyvips.tex', u'pyvips Documentation',
     u'john', 'manual'),
]


# -- Options for manual page output ---------------------------------------

# One entry per manual page. List of tuples
# (source start file, name, description, authors, manual section).
man_pages = [
    (master_doc, 'pyvips', u'pyvips Documentation',
     [author], 1)
]


# -- Options for Texinfo output -------------------------------------------

# Grouping the document tree into Texinfo files. List of tuples
# (source start file, target name, title, author,
#  dir menu entry, description, category)
texinfo_documents = [
    (master_doc, 'pyvips', u'pyvips Documentation',
     author, 'pyvips', 'One line description of project.',
     'Miscellaneous'),
]


# see https://stackoverflow.com/questions/20569011
# adds autoautosummary directive, see vimage.rst

# try to exclude deprecated
def skip_deprecated(app, what, name, obj, skip, options):
    if hasattr(obj, "func_dict") and "__deprecated__" in obj.func_dict:
        print("skipping " + name)
        return True
    return skip or False


def setup(app):
    app.connect('autodoc-skip-member', skip_deprecated)
    try:
        from sphinx.ext.autosummary import Autosummary
        from sphinx.ext.autosummary import get_documenter
        from docutils.parsers.rst import directives
        from sphinx.util.inspect import safe_getattr

        class AutoAutoSummary(Autosummary):

            option_spec = {
                'methods': directives.unchanged,
                'attributes': directives.unchanged
            }

            required_arguments = 1

            @staticmethod
            def get_members(obj, typ, include_public=None):
                if not include_public:
                    include_public = []
                items = []
                for name in dir(obj):
                    try:
                        documenter = get_documenter(safe_getattr(obj, name),
                                                    obj)
                    except AttributeError:
                        continue
                    if documenter.objtype == typ:
                        items.append(name)
                public = [x for x in items
                          if x in include_public or not x.startswith('_')]
                return public, items

            def run(self):
                clazz = str(self.arguments[0])
                try:
                    (module_name, class_name) = clazz.rsplit('.', 1)
                    m = __import__(module_name, globals(), locals(),
                                   [class_name])
                    c = getattr(m, class_name)
                    if 'methods' in self.options:
                        _, methods = self.get_members(c,
                                                      'method', ['__init__'])

                        self.content = ["~%s.%s" % (clazz, method)
                                        for method in methods
                                        if not method.startswith('_')]
                    if 'attributes' in self.options:
                        _, attribs = self.get_members(c, 'attribute')
                        self.content = ["~%s.%s" % (clazz, attrib)
                                        for attrib in attribs
                                        if not attrib.startswith('_')]
                finally:
                    return super(AutoAutoSummary, self).run()

        app.add_directive('autoautosummary', AutoAutoSummary)
    except BaseException as e:
        raise e



#!/usr/bin/python3

import sys
import pyvips

image = pyvips.Image.new_from_file(sys.argv[1], access="sequential")
watermark = pyvips.Image.new_from_file(sys.argv[3], access="sequential")

# downsize the image by 50%
image = image.resize(0.5)

# set the watermark alpha to 20% (multiply A of RGBA by 0.2).
watermark *= [1, 1, 1, 0.2]

# overlay the watermark at the bottom left, with a 100 pixel margin
image = image.composite(watermark, "over",
                        x=100, y=image.height - watermark.height - 100)

image.write_to_file(sys.argv[2])



#!/usr/bin/python3

# Shepard's distortion, from https://github.com/tourtiere/light-distortion
# with kind permission.

# Use a set of control points to distort an image.
# Usage:
#   ./shepard.py ~/pics/k2.jpg x.jpg "300,400 300,500  300,1500 300,1100"


import re
import sys
from typing import Tuple, List

import pyvips

Point = Tuple[int, int]
Couple = Tuple[Point, Point]


def shepards(image: pyvips.Image, couples: List[Couple]) -> pyvips.Image:
    """Shepard's distortion.

    Distort an image with a set of control points.
    """

    index = pyvips.Image.xyz(image.width, image.height)
    deltas = []
    weights = []
    for p1, p2 in couples:
        diff = index - list(p2)

        distance = (diff[0]**2 + diff[1]**2)
        distance = distance.ifthenelse(distance, 0.1)

        weight = 1.0 / distance

        delta = [(p1[0] - p2[0]), (p1[1] - p2[1])] * weight

        weights.append(weight)
        deltas.append(delta)

    # add, normalize
    index += pyvips.Image.sum(deltas) / pyvips.Image.sum(weights)

    return image.mapim(index, interpolate=pyvips.Interpolate.new('bicubic'))


if __name__ == '__main__':
    image = pyvips.Image.new_from_file(sys.argv[1])
    matches = re.findall('(\d+),(\d+) (\d+),(\d+)', sys.argv[3])
    couples = [((int(m[0]), int(m[1])), (int(m[2]), int(m[3])))
               for m in matches]

    image = shepards(image, couples)

    image.write_to_file(sys.argv[2])



#!/usr/bin/python3

import sys
import time

import pyvips
from PIL import Image
import numpy as np

if len(sys.argv) != 3:
    print('usage: {0} input-filename output-filename'.format(sys.argv[0]))
    sys.exit(-1)

# map vips formats to np dtypes
format_to_dtype = {
    'uchar': np.uint8,
    'char': np.int8,
    'ushort': np.uint16,
    'short': np.int16,
    'uint': np.uint32,
    'int': np.int32,
    'float': np.float32,
    'double': np.float64,
    'complex': np.complex64,
    'dpcomplex': np.complex128,
}

# map np dtypes to vips
dtype_to_format = {
    'uint8': 'uchar',
    'int8': 'char',
    'uint16': 'ushort',
    'int16': 'short',
    'uint32': 'uint',
    'int32': 'int',
    'float32': 'float',
    'float64': 'double',
    'complex64': 'complex',
    'complex128': 'dpcomplex',
}


# numpy array to vips image
def numpy2vips(a):
    height, width, bands = a.shape
    linear = a.reshape(width * height * bands)
    vi = pyvips.Image.new_from_memory(linear.data, width, height, bands,
                                      dtype_to_format[str(a.dtype)])
    return vi


# vips image to numpy array
def vips2numpy(vi):
    return np.ndarray(buffer=vi.write_to_memory(),
                      dtype=format_to_dtype[vi.format],
                      shape=[vi.height, vi.width, vi.bands])


# load with PIL
start_pillow = time.time()
pillow_img = np.asarray(Image.open(sys.argv[1]))
print('Pillow Time:', time.time() - start_pillow)
print('pil shape', pillow_img.shape)

# load with vips to a memory array
start_vips = time.time()
img = pyvips.Image.new_from_file(sys.argv[1])
np_3d = vips2numpy(img)

print('Vips Time:', time.time() - start_vips)
print('vips shape', np_3d.shape)

# make a vips image from the numpy array
vi = numpy2vips(pillow_img)

# verify we have the same result
# this can be non-zero for formats like jpg if the two libraries are using
# different libjpg versions ... try with png instead
print('Average pil/vips difference:', (vi - img).avg())

# and write back to disc for checking
vi.write_to_file(sys.argv[2])



"""a binding for the libvips image processing library

See:
https://github.com/libvips/pyvips
"""

# flake8: noqa

import sys
from codecs import open
from os import path

from setuptools import setup, find_packages
from distutils import log

here = path.abspath(path.dirname(__file__))

info = {}
with open(path.join(here, 'pyvips', 'version.py'), encoding='utf-8') as f:
    exec(f.read(), info)

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

# See https://pypi.python.org/pypi?%3Aaction=list_classifiers
pyvips_classifiers = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Console',
    'Intended Audience :: Developers',
    'Intended Audience :: Science/Research',
    'Topic :: Multimedia :: Graphics',
    'Topic :: Multimedia :: Graphics :: Graphics Conversion',
    'License :: OSI Approved :: MIT License',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.3',
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Programming Language :: Python :: Implementation :: CPython',
]

setup_deps = [
    'cffi>=1.0.0',
]

install_deps = [
    'cffi>=1.0.0',
]

test_deps = [
    'cffi>=1.0.0',
    'pytest-runner',
    'pytest',
    'pytest-flake8',
    'pyperf',
]

extras = {
    'test': test_deps,
    'doc': ['sphinx', 'sphinx_rtd_theme'],
}

pyvips_packages = find_packages(exclude=['docs', 'tests', 'examples'])

sys.path.append(path.join(here, 'pyvips'))

def setup_API():
    setup(
        name='pyvips',
        version=info['__version__'],
        description='binding for the libvips image processing library, API mode',
        long_description=long_description,
        url='https://github.com/libvips/pyvips',
        author='John Cupitt',
        author_email='jcupitt@gmail.com',
        license='MIT',
        classifiers=pyvips_classifiers,
        keywords='image processing',

        packages=pyvips_packages,
        setup_requires=setup_deps + ['pkgconfig'],
        cffi_modules=['pyvips/pyvips_build.py:ffibuilder'],
        install_requires=install_deps + ['pkgconfig'],
        tests_require=test_deps,
        extras_require=extras,

        # we will try to compile as part of install, so we can't run in a zip
        zip_safe=False,
    )

def setup_ABI():
    setup(
        name='pyvips',
        version=info['__version__'],
        description='binding for the libvips image processing library, ABI mode',
        long_description=long_description,
        url='https://github.com/libvips/pyvips',
        author='John Cupitt',
        author_email='jcupitt@gmail.com',
        license='MIT',
        classifiers=pyvips_classifiers,
        keywords='image processing',

        packages=pyvips_packages,
        setup_requires=setup_deps,
        install_requires=install_deps,
        tests_require=test_deps,
        extras_require=extras,
    )

# try to install in API mode first, then if that fails, fall back to ABI

# API mode requires a working C compiler plus all the libvips headers whereas
# ABI only needs the libvips shared library to be on the system

try:
    setup_API()
except Exception as e:
    log.warn('Falling back to ABI mode. Details: {0}'.format(e))
    setup_ABI()



#!/usr/bin/python3

import sys

import pyvips

# import logging
# logging.basicConfig(level = logging.DEBUG)

# pyvips.cache_set_trace(True)

a = pyvips.Image.new_from_file(sys.argv[1])

a = a[1:]

a.write_to_file(sys.argv[2])



#!/usr/bin/python

# import logging
# logging.basicConfig(level = logging.DEBUG)

import pyvips

a = pyvips.Image.black(100, 100)
b = a.bandjoin(2)

b.write_to_file("x.v")

txt = pyvips.Image.text("left corner", dpi=300)

c = txt.ifthenelse(2, [0, 255, 0], blend=True)

c.write_to_file("x2.v")
